package com.jzlife.report.aop;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jzlife.report.config.mvc.RequestWrapper;
import com.jzlife.report.service.newreport.PrivilegeManService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.http.MimeHeaders;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.Objects;
import java.util.stream.Collectors;

@Aspect
@Slf4j
@Component
@Order(3)
public class PrivilegeManAspect {

    @Autowired
    private PrivilegeManService privilegeManService;

    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Pointcut("execution(public * com.jzlife.report.controller.InvestmentReportController.investmentAndResignList(..)) ||" +
            "execution(public * com.jzlife.report.controller.InvestmentReportController.dayInvestMenDetail(..)) ||" +
            "execution(public * com.jzlife.report.controller.InvestmentReportController.getResignAndLossReason(..)) ||" +
            " execution(public * com.jzlife.report.controller.ServicePowerReportController.*(..)) ||" +
            " execution(public * com.jzlife.report.controller.ShopReferenceController.*(..)) ||" +
            " execution(public * com.jzlife.report.controller.ServiceQualityReportController.serviceQualityStackLine(..))||" +
            " execution(public * com.jzlife.report.controller.ServiceQualityReportController.dayCustomerClaimDetailExport(..))||" +
            " execution(public * com.jzlife.report.controller.ServiceQualityReportController.dayCustomerClaimDetail(..))||" +
            " execution(public * com.jzlife.report.controller.ServiceQualityReportController.customerClaimAndServicePowerLineGraph(..))||" +
            " execution(public * com.jzlife.report.controller.ServiceQualityReportController.onlineChatSummary(..))||" +
            " execution(public * com.jzlife.report.controller.SalesDataReportController.bdInvertDetailExport(..))||" +
            "execution(public * com.jzlife.report.controller.SalesDataReportController.bdInvertDetail(..) )")
    public void controllerAspect() {
    }

    @Before("controllerAspect()")
    public void before() {
        HttpServletRequest request = ((ServletRequestAttributes) Objects.requireNonNull(RequestContextHolder.getRequestAttributes()))
                .getRequest();
        String token = request.getHeader("token");
        if (StringUtils.isEmpty(token)) {
            token = request.getParameter("token");
        }
        String userInfo = stringRedisTemplate.opsForValue().get(token);
        JSONObject o = Objects.requireNonNull(JSON.parseArray(userInfo)).getJSONObject(1);
        Integer userId = o.getInteger("id");
        boolean flag = privilegeManService.getPrivilegeManByUserId(userId);
        reflectSetHeader(request, "userId", userId.toString());
        try {
            request.setAttribute("userId", userId);
            if (null != request.getHeader("Content-Type") && request.getHeader("Content-Type").contains("multipart/form-data")) {
                log.info("userId=" + userId + "上传文件，跳过校验城市！");
                return;
            }
            if (request.getMethod().equals("POST")) {
                String test = request.getReader().lines().collect(Collectors.joining(System.lineSeparator()));
                JSONObject jsonObject = JSON.parseObject(test);
                if (!flag & (!jsonObject.containsKey("city") || StringUtils.isEmpty(jsonObject.getString("city")))) {
//                    throw new BusinessException("请选择一个城市");
                    log.warn("userId=" + userId + "不是特权人士，只能查看他拥有的城市");

                } else if (flag) {
                    log.info("userId=" + userId + "是特权人士，可以查看所有城市");
                } else {
                    log.info("userId=" + userId + "不是特权人士，查看城市为" + jsonObject.getString("city"));
                }
            }
            reflectSetHeader(request, "allowCity", privilegeManService.getPrivilegeManCityByUserId(userId));
        } catch (IOException e) {
            e.printStackTrace();
        }


    }

    private void reflectSetHeader(HttpServletRequest request, String key, String value) {
        Class<? extends ServletRequest> requestClass = ((RequestWrapper) request).getRequest().getClass();
        try {
            Field requestField = requestClass.getDeclaredField("request");
            requestField.setAccessible(true);
            Object requestObj = requestField.get(((RequestWrapper) request).getRequest());
            Field coyoteRequestField = requestObj.getClass().getDeclaredField("coyoteRequest");
            coyoteRequestField.setAccessible(true);
            Object coyoteRequestObj = coyoteRequestField.get(requestObj);
            Field headersField = coyoteRequestObj.getClass().getDeclaredField("headers");
            headersField.setAccessible(true);
            MimeHeaders headersObj = (MimeHeaders) headersField.get(coyoteRequestObj);
            headersObj.removeHeader(key);
            headersObj.addValue(key).setString(value);
        } catch (Exception e) {
            System.out.println("设置请求头失败，失败原因：" + e.getMessage());
        }
    }

}
