package com.jzlife.report.controller;

import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.manager.CallTaskMapper;
import com.jzlife.report.dao.reports.CrmCampaignMapper;
import com.jzlife.report.dto.ConsultTypeDetailReport;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.util.List;

@Slf4j
@Api(value = "电访任务", description = "ob电访任务")
@RestController
@RequestMapping("callTask")
public class CallTaskController {

    @Autowired
    private CrmCampaignMapper crmCampaignMapper;

    @Autowired
    private CallTaskMapper callTaskMapper;


    @ApiOperation(value = "获取所有外拨任务名称", httpMethod = "GET", notes = "查询所有任务")
    @GetMapping("getAllCampaign")
    @LogRecordFiled(funName = "获取所有任务名称")
    public Object getAllCampaign() {
        return ResponseUtil.ok(crmCampaignMapper.selectAll());
    }

    @ApiOperation(value = "电访任务报表", httpMethod = "GET", notes = "电访任务报表")
    @GetMapping("getCallTask")
    @LogRecordFiled(funName = "电访任务报表")
    public Object getCallTask(String startTime, String endTime, Integer campaignId) {
        return ResponseUtil.ok(callTaskMapper.selectCallTask(startTime, endTime, campaignId));
    }

    @ApiOperation(value = "IB电访结果报表", httpMethod = "GET", notes = "IB电访结果报表")
    @GetMapping("getCallIB")
    @LogRecordFiled(funName = "IB电访结果报表")
    public Object getCallIB(String startTime, String endTime) {
        return ResponseUtil.ok(callTaskMapper.selectIB(startTime, endTime));
    }

    @ApiOperation(value = "电访工单咨询/投诉项明细报表", httpMethod = "GET", notes = "电访任务工单明细报表")
    @LogRecordFiled(funName = "电访工单明细报表")
    @GetMapping("/ticketConsultDetailList")
    public Object ticketConsultDetailList(@RequestParam(required = false) Integer campaignId,
                                          @NotNull @RequestParam(required = false) String startTime,
                                          @NotNull @RequestParam(required = false) String endTime) {

        Long days = LocalDate.parse(endTime).toEpochDay() - LocalDate.parse(startTime).toEpochDay();
        if (days > 61)
            return ResponseUtil.fail(3, "查询时间不能大于二个月");

        List<ConsultTypeDetailReport> consultTypeData = callTaskMapper.ticketConsultTypeDetailReport(campaignId, startTime, endTime);

        return ResponseUtil.ok(consultTypeData);
    }
}
