package com.jzlife.report.service.hmorder.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import com.jzlife.report.dao.hmorder.ReportEvaluationMapper;
import com.jzlife.report.dao.hmorder.ReportRebuyMapper;
import com.jzlife.report.dao.hmorder.ReportSignInMapper;
import com.jzlife.report.entity.hmorder.ReportEvaluation;
import com.jzlife.report.entity.hmorder.ReportRebuy;
import com.jzlife.report.entity.hmorder.ReportSignIn;
import com.jzlife.report.service.hmorder.ImportExcelService;
import com.jzlife.report.util.DateUtil;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
public class ImportExcelServiceImpl implements ImportExcelService {

	@Autowired
	ReportEvaluationMapper evaluationMapper;

	@Autowired
	ReportRebuyMapper reportRebuyMapper;

	@Autowired
	ReportSignInMapper reportSignInMapper;

	/**
	 * 导入生活服务评价咨询
	 * 
	 * @param datas
	 */
	@Async
	public void insertReportEvaluation(List<ReportEvaluation> datas) {
		if (null == datas || datas.size() == 0) {
			return;
		}

		String batchNo = System.currentTimeMillis() + "";
		Date now = new Date();
		List<ReportEvaluation> tempList = new ArrayList<ReportEvaluation>();

		for (ReportEvaluation ev : datas) {
			ev.setBatchNo(batchNo);
			ev.setInsertTime(now);

			if (null != ev.getTaoId()) {
				ev.setTaoId(ev.getTaoId().replace("淘", ""));
			}

			tempList.add(ev);
			if (tempList.size() > 500) {
				evaluationMapper.batchInsert(tempList);
				tempList.clear();
			}
		}

		if (tempList.size() > 0) {
			evaluationMapper.batchInsert(tempList);
		}

		log.info(batchNo + "服务评价插入完毕，多少条：" + datas.size());

		//以统计日期做去重
		int deleteBatchDate = evaluationMapper.deleteBatchDate(batchNo);

		log.info("删除重复数据，服务评价，多少条：" + deleteBatchDate);
	}

	/**
	 * 导入复购
	 */
	@Override
	public void insertReportRebuy(List<ReportRebuy> datas) {
		if (null == datas || datas.size() == 0) {
			return;
		}

		Date now = new Date();
		List<ReportRebuy> tempList = new ArrayList<ReportRebuy>();

		for (ReportRebuy ev : datas) {
			ev.setCreateTime(now);
			tempList.add(ev);
			if (tempList.size() > 500) {
				reportRebuyMapper.batchInsert(tempList);
				tempList.clear();
			}
		}

		if (tempList.size() > 0) {
			reportRebuyMapper.batchInsert(tempList);
		}

		log.info("导入复购插入完毕，多少条：" + datas.size());

		//付款单号去重
		int ct = reportRebuyMapper.deleteRepeatPayNo();

		log.info("删除重复付款单号数据，复购，多少条：" + ct);

	}

	@Async
	public void insertReportSignIn(List<ReportSignIn> datas) {

		if (null == datas || datas.size() == 0) {
			return;
		}

		String batchNo = System.currentTimeMillis() + "";
		Date now = new Date();
		List<ReportSignIn> tempList = new ArrayList<ReportSignIn>();

		for (ReportSignIn ev : datas) {
			ev.setBatchNo(batchNo);
			ev.setCreateTime(now);
			// 时间处理
			ev.setStartTime(getConvertVal(ev.getStartTime2()));
			ev.setOndoorTime(getConvertVal(ev.getOndoorTime2()));

			ev.setEndTime(getConvertVal(ev.getEndTime2()));
			ev.setAllprovideTime(getConvertVal(ev.getAllprovideTime2()));

			tempList.add(ev);
			if (tempList.size() > 500) {
				reportSignInMapper.batchInsert(tempList);
				tempList.clear();
			}
		}

		if (tempList.size() > 0) {
			reportSignInMapper.batchInsert(tempList);
		}

		log.info(batchNo + "打卡记录插入完毕，多少条：" + datas.size());

		//预约开始时间去重
		int deleteBatchDate = evaluationMapper.deleteBatchDate(batchNo);

		log.info("删除重复数据，打卡记录，多少条：" + deleteBatchDate);

	}

	public Date getConvertVal(String date) {
		if (null == date || date.equals("-")) {
			return null;
		}

		try {
			if(date.length()>16) {
				return DateUtil.getDate(date);
			}
			return DateUtil.getDate2(date);
//			Date d=DateUtil.getDate(date);
//			if(d!=null) {
//				return d;
//			}
//			return DateUtil.getDate2(date);
		} catch (Exception e) {

		}

		return null;
	}

}
