package com.jzlife.report.vo;


import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
public class DayReportBaseVo {

    @ApiModelProperty(value = "城市名称", notes = "支持多个城市名称，多个城市名称用【,】分开")
    private String city;


    @ApiModelProperty(value = "查询开始日期", notes = "时间格式为 yyyy-MM-dd")
    @NotNull(message = "查询开始日期不可为空")
    @Pattern(regexp = "^20(19|2\\d)-(0[1-9]|1[012])-(0[1-9]|1[0-9]|2[0-9]|3[0-1])$", message = "开始时间不合法")
    private String startTime;

    @ApiModelProperty(value = "查询结束日期", notes = "时间格式为 yyyy-MM-dd")
    @NotNull(message = "查询结束日期不可为空")
    @Pattern(regexp = "^20(19|2\\d)-(0[1-9]|1[012])-(0[1-9]|1[0-9]|2[0-9]|3[0-1])$", message = "结束时间不合法")
    private String endTime;


    @ApiModelProperty(value = "服务站名称", notes = "服务站名称")
    private String serviceStation;

    @ApiModelProperty(value = "服务者姓名", notes = "服务者姓名")
    private String serverName;

    @ApiModelProperty(value = "客诉类型", notes = "客诉类型 1 咨询 2投诉")
    private Integer claimType;

    @ApiModelProperty(value = "客诉咨询项", notes = "客诉2级，仅服务质量-客诉明细")
    private Integer[] consultValue;


    @ApiModelProperty(value = "客诉等级", notes = "一般、严重、S")
    private Integer customerClaimLevel;


    @ApiModelProperty(value = "客诉编号", notes = "")
    private String ticketSn;

    @ApiModelProperty(value = "客诉状态", notes = "客诉状态  1 处理中 2已完成   3 已作废 99超48小时未完结")
    private Integer claimStatus;

    @ApiModelProperty(value = "开始页", notes = "开始页")
    private Integer startPage = 1;
    @ApiModelProperty(value = "每页展示条数", notes = "每页展示条数")
    private Integer limit = 50;

    @ApiModelProperty(value = "服务者姓名")
    private String workerName;
}
